/*
 Solarbotics Firefly Count-down Timer Code
 
 This example code shows how to use the Firefly as a 3 min Count-down Timer.
 The code is also a great example on how to use interrupts and sleep functions with the ATTINY45/85V at 8MHz.
 
 The circuit:
 * LED1 is attached from digital pin 3 (D4) to the 47 ohm resistot then to GND
 * LED2 is attached from digital pin 6 (D1) to the 47 ohm resistor then to GND
 * LED3 is attached from digital pin 5 (D0) to the 47 ohm resistor then to GND
 * An additional Piezo buzzer is added from pin 2 (D3) [PB3 on the expansion port] to GND 
 * The button is attached from digital pin 7 (D2) to GND
 
 Created June 2012
 By Jeremie Boulianne for Solarbotics Ltd.
 
 Firefly inspiration and sponsorship from Awesome Calgary winner:
 
 Calgary's Illuminated Landscapes: http://www.collaborativeprojectsyyc.blogspot.ca/
 
 Code elements & information borrowed from the following resources:
 
 High Low Tech's Programming an ATtiny with Arduino 1.0 Tutorial: http://hlt.media.mit.edu/?p=1695
 Engblaze's AVR/Arduino sleep function tutorial: http://www.engblaze.com/hush-little-microprocessor-avr-and-arduino-sleep-mode-basics/
 Engblaze's AVR/Arduino interrupt tutorial: http://www.engblaze.com/we-interrupt-this-program-to-bring-you-a-tutorial-on-arduino-interrupts/ 
 Alex Iannuzzi's reducing ATtiny power tutorial : http://www.insidegadgets.com/2011/02/05/reduce-attiny-power-consumption-by-sleeping-with-the-watchdog-timer/
 Dale Wheat's Tiny cyclon project: http://www.dalewheat.com/info/
 */

#include <avr/sleep.h>

//Code needed to modify the Analog to Digital Convertor (ADC) register bits to turn it on and off
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif                                             

int wakePin = 0;                           //Interrupt 0 which is the button on DIGITAL PIN 2!
int mode = 0;                              //Case value for switch statement 
volatile int btnPress = 0;                 //Button Press which must be called as a volatile variable because it is used in an Interrupt Service Routine

void setup() {
  pinMode(3, OUTPUT);                      //Create (D3) Piezo Buzzer as an output
  pinMode(4, OUTPUT);                      //Create (D4) LED1 as an output
  pinMode(1, OUTPUT);                      //Create (D1) LED2 as an output 
  pinMode(0, OUTPUT);                      //Create (D0) LED3 as an output 
  pinMode(2, INPUT);                       //Create our button pin as an input on D2
  digitalWrite(2, HIGH);                   //Enable pullup resistor for button so it isn't floating
  attachInterrupt(wakePin, wakeUp, LOW);   //Attach the interrupt to the button on D2 and execute the ISR on a LOW signal
}

void loop(){

  switch(mode){                             //Switch case statement which check for which mode the program needs to be in

    case 0: 
      digitalWrite(1, LOW);
      digitalWrite(4, LOW);
      digitalWrite(0, HIGH);                //Turn LED3 ON and others Off
      Pause(6000);                          //Pause 1 minute
      digitalWrite(1, HIGH);                //Turn LED2 ON 
      Beep(2000);                           //400mS beep 
      Pause(6000);                          //Pause 1 minute
      digitalWrite(4, HIGH);                //Turn LED1 ON
      Beep(2000);                           //400mS beep 
      Pause(40);                            //400 mS Pause
      Beep(2000);                           //400mS beep
      Pause(6000);                          //Pause 1 minute
      
        if (btnPress > 0)  {                //Watch for button press
          delay(500);                       //Debounce the button
          mode = 0;                         //Reset Switch Case value
          btnPress = 0;                     //Reset button interrupt value
          break; 
        }
      mode++;                               //Increase the case value to move to next section
      break;

    case 1:

      while(1){                             //Continuously Flash the LEDs and Beep the Piezo until the Button is pressed

        if (btnPress > 0)  {                //Watch for button press
          sleepNow();                       //Put Firefly to sleep
          delay(500);                       //Debounce the button
          mode = 0;                         //Reset Switch Case value
          btnPress = 0;                     //Reset button interrupt value
          break; 
        }

        for (int i = 0; i < 2000; i++) { 
          digitalWrite(0, HIGH); 
          digitalWrite(1, HIGH);
          digitalWrite(4, HIGH);            //Turn ON all the LEDs
          digitalWrite(3, HIGH);
          delayMicroseconds(100);
          digitalWrite(3, LOW);
          delayMicroseconds(100);           //Generate a vibration on the Piezo element

            if (btnPress > 0)  {            //Watch for button press
              delay(500);                   //Debounce the button
              break;
            }
         }

        if (btnPress > 0)  {                //Watch for button press
          sleepNow();                       //Put Firefly to sleep
          delay(500);                       //Debounce the button
          mode = 0;                         //Reset Switch Case value
          btnPress = 0;                     //Reset button interrupt value
          break; 
        }


        for (int m = 750; m > 0; m--) {
          digitalWrite(0, LOW); 
          digitalWrite(1, LOW);
          digitalWrite(4, LOW);             //Turn OFF all the LEDs

            if (btnPress > 0)  {            //Watch for button press
              delay(500);                   //Debounce the button
              break; 
             }
          delay(1); 
        }
      }
  }  
}

/*************************WAKE UP SUBROUTINE***************************************/

void wakeUp(){                             //ISR -> Interrupt Service Routine, called when button is pressed

  btnPress = 1;                            //Change the state of the Button Press in order to exit whatever loop is running
}

/*************************SLEEP NOW SUBROUTINE***************************************/

void sleepNow(){                            //A subroutine that puts the Arduino to sleep and turns off the ADC

  digitalWrite(0, LOW); 
  digitalWrite(1, LOW);
  digitalWrite(4, LOW);                    //Turn off all the LEDs

  cbi(ADCSRA,ADEN);                        //Switch ADC OFF

  set_sleep_mode(SLEEP_MODE_PWR_DOWN);     //Sleep mode is set here

  sleep_enable();                          //Enables the sleep bit in the mcucr register so sleep is possible. just a safety pin

  sleep_mode();                            //Here the device is actually put to sleep!! THE PROGRAM CONTINUES FROM HERE AFTER WAKING UP FROM INTERRUT

  sleep_disable();                         //First thing after waking from sleep: disable sleep

  sbi(ADCSRA,ADEN);                        //Switch ADC ON
}

/*************************PAUSE SUBROUTINE***************************************/

void Pause(unsigned int a){                //A subroutine to check for a button press during a delay                         

  for (int m = a; m > 0; m--) {

    if (btnPress > 0)  {                   //Watch for button press
      break; 
    }
    delay(10); 
  }                            
}


/*************************BEEP SUBROUTINE***************************************/

void Beep(unsigned int b){                 //A subroutine to check for a button press while generating a beep alarm on the piezo buzzer                   

  for (int i = 0; i < b; i++) {  
    digitalWrite(3, HIGH);
    delayMicroseconds(100);
    digitalWrite(3, LOW);
    delayMicroseconds(100);               //Generate a vibration on the Piezo element 

    if (btnPress > 0)  {                  //Watch for button press
      break;
    }                        
  }
}

